require 'test/unit'
require 'testcase'
require 'fox12'

include Fox

class OverrideError < Exception
end

class CustomTable < FXTable
  def setColumnWidth(col, cwidth)
    raise OverrideError
  end

  def setRowHeight(row, rheight)
    raise OverrideError
  end

  def setColumnX(colEdge, x)
    raise OverrideError
  end

  def setRowY(rowEdge, y)
    raise OverrideError
  end
end

class TC_FXTable < TestCase

private

  def populateTable
    @table.each_row do |row|
      row.each { |item| item.text = "foo" }
    end
  end

  def clearEdges(extending=false)
    item = nil
    nr = @table.numRows
    nc = @table.numColumns
    
    (1..nc-2).each { |c|
      item = @table.getItem(nr-1, c)
        item.button = false
        item.justify = FXTableItem::RIGHT
    }
    
    unless extending
      (1..nr-2).each { |r|
        item = @table.getItem(r, nc-1)
          item.button = false
      }
      if nr > 0 and nc > 0
        item = @table.getItem(nr-1, nc-1)
          item.button = false
          item.text = ""
      end
    end
  end

  def loadLogChunk(num_rows)
    if @logsource
      extending = true
    else
      extending = false
      @logsource = nil
      @header_row = ["foo"]*10
      @data_lines = []
    end

    100.times { @data_lines << ["1"]*10 }

    item = nil

    clearEdges(extending)

    old_nr = @table.numRows
    old_nc = @table.numColumns

    # resize the table
    nr = @data_lines.size + 2
    nc = @header_row.size + 2
    @table.setTableSize(nr, nc)

    @table.leadingRows = 1
    @table.leadingColumns = 1
    @table.trailingRows = 1
    @table.trailingColumns = 1

    # Initialize first & last fixed rows
    (1..nc-2).each { |c|
      unless extending
        @table.setItemText(0, c, @header_row[c-1])
        item = @table.getItem(0, c)
          item.button = true
      end
      @table.setItemText(nr-1, c, @header_row[c-1])
      item = @table.getItem(nr-1, c)
        item.button = true
    }

    # Initialize first & last fixed columns
    start_r = extending ? old_nr-1 : 1
    (start_r..nr-2).each { |r|
      @table.setItemText(r,  0, "#{r}")
      @table.setItemText(r, nc-1, "#{r}")
      @table.getItem(r, 0).setButton(true)
      @table.getItem(r, nc-1).setButton(true)
    }

    # The corners are just buttons
    @table.setItemText(0, 0, "")
    @table.getItem(0, 0).setButton(true)
    @table.setItemText(0, nc-1, "")
    @table.getItem(0, nc-1).setButton(true)
    @table.setItemText(nr-1, 0, "")
    @table.getItem(nr-1, 0).setButton(true)
    @table.setItemText(nr-1, nc-1, "")
    item = @table.getItem(nr-1, nc-1)
      item.button = true
      if false
        # disable the button
        @more_pos = nil
      else
        remaining = 100
        if remaining > 1024
          item.text = "%5.1fM MORE" % (remaining/1024)
        else
          item.text = "%5.1fK MORE" % remaining
        end
        @more_pos = FXTablePos.new
        @more_pos.row = nr-1; @more_pos.col = nc-1
      end
    @table.setItemText(0, nc-1, "")

    # Initialize scrollable part of table
    entry = nil
    (start_r..nr-2).each { |r|
      (1..nc-2).each { |c|
        entry = @data_lines[r-1][c-1]
        case entry
        when /\A(.*\.)(.*)\z/
          entry = sprintf("%10.5f", entry.to_f).
            sub(/\.(\d*?)(0*)$/) { ".#{$1}#{$2.gsub("0", " ")}" }
        when /\d+/
          entry << " " * 6
        end
        @table.setItemText(r, c, entry)
      }
    }
  end

  # Load the named log file
  def loadLog filename = @file_names
    @file_names = filename
    @logsource = nil
    clearEdges
    @table.setTableSize(0, 0)
    loadLogChunk(100)
  end

public
  
  def setup
    super(self.class.name)
    @table = FXTable.new(mainWindow, 0, 0)
    @customTable = CustomTable.new(mainWindow, 0, 0)
    populateTable
  end
  
  def test_setTableSize
    100.times { loadLog } # this should be enough to do it
  end
  
  def test_getCellColor
    assert_nothing_raised {
      @table.getCellColor(0, 0)
    }
    assert_nothing_raised {
      @table.getCellColor(0, 1)
    }
    assert_nothing_raised {
      @table.getCellColor(1, 0)
    }
    assert_nothing_raised {
      @table.getCellColor(1, 1)
    }

    @table.setTableSize(5, 5)
    assert_raises(IndexError) {
      @table.getCellColor(-1, 0)
    }
    assert_raises(IndexError) {
      @table.getCellColor(2, 0)
    }
    assert_raises(IndexError) {
      @table.getCellColor(0, -1)
    }
    assert_raises(IndexError) {
      @table.getCellColor(0, 2)
    }
  end

  def test_setCellColor
    assert_nothing_raised {
      @table.setCellColor(0, 0, FXRGB(0, 0, 0))
    }
    assert_nothing_raised {
      @table.setCellColor(0, 1, FXRGB(0, 0, 0))
    }
    assert_nothing_raised {
      @table.setCellColor(1, 0, FXRGB(0, 0, 0))
    }
    assert_nothing_raised {
      @table.setCellColor(1, 1, FXRGB(0, 0, 0))
    }

    @table.setTableSize(5, 5)
    assert_raises(IndexError) {
      @table.setCellColor(-1, 0, FXRGB(0, 0, 0))
    }
    assert_raises(IndexError) {
      @table.setCellColor(2, 0, FXRGB(0, 0, 0))
    }
    assert_raises(IndexError) {
      @table.setCellColor(0, -1, FXRGB(0, 0, 0))
    }
    assert_raises(IndexError) {
      @table.setCellColor(0, 2, FXRGB(0, 0, 0))
    }
  end
  
  def test_updateRange
    @table.setTableSize(5, 5)
    assert_nothing_raised {
      @table.updateRange(0, 5, 0, 5)
    }
    assert_raises(IndexError) {
      @table.updateRange(-1, 5, 0, 5) # startRow < 0
    }
    assert_raises(IndexError) {
      @table.updateRange(0, 6, 0, 5) # endRow > numRows
    }
    assert_raises(IndexError) {
      @table.updateRange(0, 5, -1, 5) # startCol < 0
    }
    assert_raises(IndexError) {
      @table.updateRange(0, 5, 0, 6) # endCol > numColumns
    }
  end
  
  def test_insertRows
    @table.setTableSize(5, 5)
    assert_nothing_raised {
      @table.insertRows(0)
    }
    assert_nothing_raised {
      @table.insertRows(@table.numRows)
    }
    assert_raises(IndexError) {
      @table.insertRows(-1) # row < 0
    }
    assert_raises(IndexError) {
      @table.insertRows(@table.numRows+1) # row > numRows
    }
  end

  def test_insertColumns
    @table.setTableSize(5, 5)
    assert_nothing_raised {
      @table.insertColumns(0)
    }
    assert_nothing_raised {
      @table.insertColumns(@table.numColumns)
    }
    assert_raises(IndexError) {
      @table.insertColumns(-1) # column < 0
    }
    assert_raises(IndexError) {
      @table.insertColumns(@table.numColumns+1) # column > numColumns
    }
  end
  
  def test_removeRows
    @table.setTableSize(8, 5)
    assert_raises(IndexError) {
      @table.removeRows(-1)
    }
    assert_nothing_raised {
      @table.removeRows(0)
    }
    assert_nothing_raised {
      @table.removeRows(@table.numRows-1)
    }
    assert_raises(IndexError) {
      @table.removeRows(@table.numRows)
    }
  end
  
  def test_removeColumns
    @table.setTableSize(5, 8)
    assert_raises(IndexError) {
      @table.removeColumns(-1)
    }
    assert_nothing_raised {
      @table.removeColumns(0)
    }
    assert_nothing_raised {
      @table.removeColumns(@table.numColumns-1)
    }
    assert_raises(IndexError) {
      @table.removeColumns(@table.numColumns)
    }
  end
  
  def test_getColumnX
    @table.setTableSize(5, 5)
    assert_raises(IndexError) {
      @table.getColumnX(-1)
    }
    assert_nothing_raised {
      @table.getColumnX(0)
    }
    assert_nothing_raised {
      @table.getColumnX(5)
    }
    assert_raises(IndexError) {
      @table.getColumnX(6)
    }
  end
  
  def test_setColumnX
    @table.setTableSize(5, 5)
    assert_raises(IndexError) {
      @table.setColumnX(-1, 0)
    }
    assert_nothing_raised {
      @table.setColumnX(0, 0)
    }
    assert_nothing_raised {
      @table.setColumnX(5, 0)
    }
    assert_raises(IndexError) {
      @table.setColumnX(6, 0)
    }
  end
  
  def test_getRowY
    @table.setTableSize(5, 5)
    assert_raises(IndexError) {
      @table.getRowY(-1)
    }
    assert_nothing_raised {
      @table.getRowY(0)
    }
    assert_nothing_raised {
      @table.getRowY(5)
    }
    assert_raises(IndexError) {
      @table.getRowY(6)
    }
  end
  
  def test_setRowY
    @table.setTableSize(5, 5)
    assert_raises(IndexError) {
      @table.setRowY(-1, 0)
    }
    assert_nothing_raised {
      @table.setRowY(0, 0)
    }
    assert_nothing_raised {
      @table.setRowY(5, 0)
    }
    assert_raises(IndexError) {
      @table.setRowY(6, 0)
    }
  end
  
  def test_setLeadingRows
    @table.setTableSize(5, 5)
    @table.trailingRows = 2
    assert_raises(IndexError) {
      @table.leadingRows = -1
    }
    assert_nothing_raised {
      @table.leadingRows = 0
    }
    assert_nothing_raised {
      @table.leadingRows = @table.numRows - @table.trailingRows
    }
    assert_raises(IndexError) {
      @table.leadingRows = @table.numRows - @table.trailingRows + 1
    }
  end
  
  def test_setLeadingCols
    @table.setTableSize(5, 5)
    @table.trailingColumns = 2
    assert_raises(IndexError) {
      @table.leadingColumns = -1
    }
    assert_nothing_raised {
      @table.leadingColumns = 0
    }
    assert_nothing_raised {
      @table.leadingColumns = @table.numColumns - @table.trailingColumns
    }
    assert_raises(IndexError) {
      @table.leadingColumns = @table.numColumns - @table.trailingColumns + 1
    }
  end
  
  def test_setTrailingRows
    @table.setTableSize(5, 5)
    @table.leadingRows = 2
    assert_raises(IndexError) {
      @table.trailingRows = -1
    }
    assert_nothing_raised {
      @table.trailingRows = 0
    }
    assert_nothing_raised {
      @table.trailingRows = @table.numRows - @table.leadingRows
    }
    assert_raises(IndexError) {
      @table.trailingRows = @table.numRows - @table.leadingRows + 1
    }
  end
  
  def test_setTrailingCols
    @table.setTableSize(5, 5)
    @table.leadingColumns = 2
    assert_raises(IndexError) {
      @table.trailingColumns = -1
    }
    assert_nothing_raised {
      @table.trailingColumns = 0
    }
    assert_nothing_raised {
      @table.trailingColumns = @table.numColumns - @table.leadingColumns
    }
    assert_raises(IndexError) {
      @table.trailingColumns = @table.numColumns - @table.leadingColumns + 1
    }
  end

  def test_overridden_setColumnWidth
    # What can we do to simulate a call to setColumnWidth?
  end

  def test_overridden_setRowHeight
    # What can we do to simulate a call to setRowHeight?
  end

  def test_overridden_setColumnX
#   assert_raises(OverrideError) {
#     @customTable.handle(nil, FXSEL(SEL_MOTION, 0), event)
#   }
  end

  def test_overridden_setRowY
#   assert_raises(OverrideError) {
#     @customTable.handle(nil, FXSEL(SEL_MOTION, 0), event)
#   }
  end
end
